@@ -141,6 +141,7 @@ typedef enum
   CONST_STRING,
   CONST_OOP,
   CONST_BINDING,
+  CONST_DEFERRED_BINDING,
   CONST_ARRAY
 }
 const_type;
@@ -180,6 +181,7 @@ typedef struct method_node
   tree_node attributes;
   tree_node statements;
   int64_t endPos;
+  mst_Boolean isOldSyntax;
 }
 method_node;
 
@@ -229,7 +231,8 @@ extern tree_node _gst_make_method (YYLTYPE *startLocation,
 				   tree_node selectorExpr,
 				   tree_node temporaries,
 				   tree_node attributes,
-				   tree_node statements)
+				   tree_node statements,
+				   int isOldSyntax)
   ATTRIBUTE_HIDDEN;
 
 /* Create an expr_node to be passed to _gst_make_method for a unary
@@ -334,6 +337,12 @@ extern tree_node _gst_make_string_constant (YYLTYPE *location,
 				  	    const char *sval)
   ATTRIBUTE_HIDDEN;
 
+/* Create a const_node storing a deferred variable binding, whose key
+   is the variable VARNODE.  */
+extern tree_node _gst_make_deferred_binding_constant (YYLTYPE *location,
+						      tree_node varNode)
+  ATTRIBUTE_HIDDEN;
+
 /* Create a const_node for an array whose elements are
    described by the nodes in the list, AVAL.  */
 extern tree_node _gst_make_array_constant (YYLTYPE *location,
